/*
 * Copyright 2011 Yuen Ho Wong
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License.  You may obtain a copy
 * of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.bitbucket.sbt_filter_plugin

import java.io.{ File, StringWriter, PrintWriter, FileReader, BufferedReader }
import java.util.{ Enumeration, Properties => JProperties }

import scala.io.Source
import scala.collection.immutable._

import sbt._

trait SBTWebappFilterPlugin extends BasicWebScalaProject with MavenStyleWebScalaPaths with SBTFilterPlugin {

  def filterWebappResourcesAction = task {
    if (filterPath.exists) {
      val webappXmlFiles = temporaryWarPath / "WEB-INF" * "*.xml" +++ temporaryWarPath / "META-INF" ** "*.xml"
      webappXmlFiles.getFiles.foreach(x => filterResource(filterSources.get, x))
    } else {
      log.warn(filterPath.toString + " is missing")
    }
    None
  } describedAs "Filter the webapp resource files for variable substitutions"
  lazy val filterWebappResources = filterWebappResourcesAction

  override lazy val prepareWebapp = filterWebappResources dependsOn (prepareWebappAction)
}

trait SBTFilterPlugin extends BasicScalaProject with MavenStyleScalaPaths {

  lazy val sfpEnv = propertyOptional[String]("dev")

  def filterResource(filterFilePaths: Iterable[Path], dest: File): Unit = {

    implicit def javaEnumeration2Iterator[A](e: Enumeration[A]) = new Iterator[A] {
      def next = e.nextElement
      def hasNext = e.hasMoreElements
    }

    def substitute(prop: JProperties, f: File) = {
      if (f.isFile) {
        val buf = new StringWriter
        val in = Source.fromFile(f)
        in.getLines.foreach(l => {
          var line = l
          prop.propertyNames.foreach(key => {
            line = line.replaceAll("\\$\\{\\s*" + key.toString + "\\s*\\}", prop.getProperty(key.toString))
          })
          buf.write(line)
        })

        val out = new PrintWriter(f)
        out.print(buf.toString)
        out.close()
      }
    }

    val envPropertyFilesMap = HashMap[String, String]() ++ filterFilePaths.map(path => path.asFile.getName.split("\\.")(0) -> path.absolutePath)

    val propFile = new BufferedReader(new FileReader(envPropertyFilesMap(sfpEnv.value)))
    val envProps = new JProperties
    try {
      envProps.load(propFile)
    } finally {
      propFile.close()
    }

    log.debug(envProps.toString)

    substitute(envProps, dest)
  }

  def filterPath = mainSourcePath / "filters"
  def filterSources = filterPath * "*.properties"

  def filterMainResourcesAction = task {
    if (filterPath.exists) {
      (mainResourcesOutputPath * "*").getFiles.foreach(x => filterResource(filterSources.get, x))
    } else {
      log.warn(filterPath.toString + " is missing")
    }
    None
  } describedAs "Filter the main resource files for variable substitutions."
  lazy val filterMainResources = filterMainResourcesAction

  def filterTestResourcesAction = task {
    if (filterPath.exists) {
      (testResourcesOutputPath * "*").getFiles.foreach(x => filterResource(filterSources.get, x))
    } else {
      log.warn(filterPath.toString + " is missing")
    }
    None
  } describedAs "Filter the test resource files for variable substitutions."
  lazy val filterTestResources = filterTestResourcesAction

  override lazy val copyResources = filterMainResources dependsOn (copyResourcesAction)
  override lazy val copyTestResources = filterTestResources dependsOn (copyTestResourcesAction)

}
